/* TestBootCDCreation.java
*
* -----------------------------------------------------------------
* Licensed Materials - Property of IBM
* 5724Q99
* (C) Copyright IBM Corp. 1998, 2006
* All Rights Reserved
* US Government Users Restricted Rights - Use, duplication or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
* -----------------------------------------------------------------
* $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestBootCDCreation.java#1 $
*/
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.util.Date;
import java.util.logging.Level;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBActivityTargetResults;
import com.ibm.rbapi.RBEvent;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.RBWebExtensionStatus;

/*
 * This example Schedules an activity to create a Bootable CD-ROM from a host 
 * running the Web Interface Extension. The bootable CD-ROM can be used to start 
 * the deployment engine on a host without using PXE. 
 */
public class TestBootCDCreation {
    public static void main(String[] args) 
        throws RBServerException, RBObjectNotFoundException, NumberFormatException, IOException, InterruptedException 
    {
        if (args.length < 3) {
            System.out.println("usage: TestBootCDCreation <ip> <port> <password>");
            return;
        }

        // Create RBServer
        System.out.println("TestBootCDCreation starting.");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret, Level.INFO); 
        System.out.println("Created rbServer.\n");

        // Select host
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        RBHostRecs hostRecs = rbServer.getHosts("", "");
        RBHost[] availHosts = hostRecs.subList(0, hostRecs.size()); 
        for (int i=0; i<availHosts.length; i++) {
            System.out.println(i + ") " + availHosts[i].getIPAddress()
                    + " MAC: " + availHosts[i].getMacAddress()
                    + " Description: " + availHosts[i].getDescription());
        }
        
        int hostIndex = -1;
        do {
            System.out.println("Please enter the number for the host running the WIE you wish to use to create the boot CD from: \n");
            hostIndex = Integer.parseInt(inReader.readLine().trim());
        } while ( hostIndex < 0 || hostIndex >= hostRecs.size() );
        
        RBHost rbhost = availHosts[hostIndex];
        
        // Check if Web Extension is ready on that host
        System.out.println("Checking RBServer.isWebExtensionReady() for host: " + rbhost.getIPAddress() + "...");
        RBWebExtensionStatus wieStatus = null;
        for (int i=0; i<10; i++) {  // try at most 10 times (20 seconds)
            wieStatus = rbServer.isWebExtensionReady(rbhost.getIPAddress());
            if (wieStatus.isPresent()) {
                break;
            }
            Thread.sleep(2000);
        }
        System.out.println("WIE - isPresent(): " + wieStatus.isPresent());
        System.out.println("WIE - getWebExtensionVersion(): " + wieStatus.getWebExtensionVersion());
        System.out.println("WIE - getWebExtensionBuild(): " + wieStatus.getWebExtensionBuild());
        System.out.println("WIE - getServerVersion(): " + wieStatus.getServerVersion());
        System.out.println("WIE - getServerBuild(): " + wieStatus.getServerBuild());
        
        if (!wieStatus.isPresent()) {
            System.out.println("The Web Interface Extension is not detected on the target host.  " +
                    "Please ensure that it is started and run this test again.");
            System.exit(1);
        }
        
        //Parameters
        System.out.println("\n\nPlease enter the filename of the boot CD ISO file." +
                "  This should be the full path to the file on the host: ");
        String isoFile = inReader.readLine().trim();
        
        System.out.println("\n\nPlease enter the IP address for the TPMfOSD server (defaults to RBServer): ");
        String serverIP = inReader.readLine().trim();

        System.out.println("\n\nPlease enter the password for the TPMfOSD server: ");
        String serverPassword = inReader.readLine().trim();

        System.out.println("\n\nDo you want to provide a fixed IP for the client (Y/N, defaults to N)? ");
        String provideFixedIP = inReader.readLine().trim();
        
        InetAddress fixedIP = null;
        InetAddress fixedNetmask = null;
        InetAddress fixedGateway = null;
        boolean allowIPOverload = false;
        boolean allowServerIPOverload = false;
        
        if (provideFixedIP.equalsIgnoreCase("Y")) {
            System.out.println("\n\nPlease enter the fixed IP address for the client? ");
            String fixedIPInput = inReader.readLine().trim();
            fixedIP = fixedIPInput.equals("") ? null : InetAddress.getByName(fixedIPInput);

            System.out.println("\n\nPlease enter the fixed subnet mask for the client? ");
            String fixedNetmaskInput = inReader.readLine().trim();
            fixedNetmask = fixedNetmaskInput.equals("") ? null : InetAddress.getByName(fixedNetmaskInput);

            System.out.println("\n\nPlease enter the fixed gateway for the client? ");
            String fixedGatewayInput = inReader.readLine().trim();
            fixedGateway = fixedGatewayInput.equals("") ? null : InetAddress.getByName(fixedGatewayInput);

            System.out.println("\n\nDo you want to overload the IP address of the client (Y/N, defauls to N)? ");
            allowIPOverload = inReader.readLine().trim().equalsIgnoreCase("Y") ? true : false;

            System.out.println("\n\nDo you want to overload the IP address of the server (Y/N, defauls to N)? ");
            allowServerIPOverload = inReader.readLine().trim().equalsIgnoreCase("Y") ? true : false;
        }
        
        
        Date now = new Date();
        Date stop = new Date(now.getTime()+1000*3600);
        Date expire = new Date(now.getTime()+1000*3600*24);
        RBActivity act = rbServer.registerBootCDCreationActivity("TestBootCdCreationActivity", 
                true, null, rbhost, isoFile, 
                serverIP.equals("") ? null : InetAddress.getByName(serverIP), serverPassword,
                fixedIP, fixedNetmask, fixedGateway, allowIPOverload, allowServerIPOverload,        
                now, stop, expire);

        Date lastTime = new Date();
        boolean foundCompletedEvent = false;
        while (!foundCompletedEvent) {
            Thread.sleep(8000);
            System.out.println("Getting all events for this activity from server from time: " + lastTime);
            RBEvent[] events =  act.getEventsFromServer(lastTime);
            RBEvent e = null;
            for (int i=0; i<events.length; i++) {
                e = events[i];

                System.out.println("Event: Progress=" + e.getProgress() + 
                        " Progress Msg=" + e.getLastProgressMsg() + 
                        " Status=" + e.getStatus());
                
                // Let's get information about events that are completed or failed
                if ( e.getStatus().equals(RBActivityTarget.COMPLETED) ||
                        e.getStatus().equals(RBActivityTarget.FAILED)) {
                    System.out.println("Getting corresponding RBActivityTarget object");
                    RBActivityTarget ourActTarget = e.getRBActivityTarget();
                    System.out.println("Activity Target: Progress=" + ourActTarget.getProgress() + 
                            " Progress Msg=" + ourActTarget.getLastProgressMsg() + 
                            " Status=" + ourActTarget.getStatus());
                    
                    // Print out the specific results for the activity target
                    RBActivityTargetResults results = ourActTarget.getActivityTargetResults();
                    System.out.println("### Results for our boot CD creation activity: ");
                    System.out.println("Status: " + results.getStatus());

                    foundCompletedEvent = true;
                    System.out.println("Exiting loop.");
                }
            }
            if (e != null) {
                lastTime = e.getUpdateTime();
            } else {
                System.out.println("No events found.");
            }
        }
        
        System.out.print("\nTestBootCDCreation done.");
    }

}
