/* TestImageManagement.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestImageManagement.java#7 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.Vector;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBActivityTargetRecs;
import com.ibm.rbapi.RBActivityTemplate;
import com.ibm.rbapi.RBActivityTemplateRecs;
import com.ibm.rbapi.RBConfiguration;
import com.ibm.rbapi.RBConfigurationRecs;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.RBSoftwarePackage;
import com.ibm.rbapi.RBSoftwarePackageRecs;
import com.ibm.rbapi.RBSystemProfile;
import com.ibm.rbapi.RBSystemProfileRecs;
import com.ibm.rbapi.exception.RBTimeoutException;

/*
 * Displays the objects known on the target server.
 * Can be used to copy objects from another server.
 * 
 * If you have multiple servers, you can see if the object in question
 * is on your server (the one you are connecting to with the java api) or not.  
 * If not, you can use this to copy the object to your server.
 */
public class TestImageManagement {

    public static void main(String[] args) 
    throws UnknownHostException, IOException, InterruptedException, 
    RBServerException, RBObjectNotFoundException, RBTimeoutException
    {
        
        if (args.length < 3 || args.length > 3) {
            System.out.println("usage: TestImageManagement <ip> <port> <password>\n");
            return;
        }

        System.out.println("TestImageManagement starting.");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret); 
        // Abort if the server is not ready after 15 seconds
        rbServer.isReady(15000);
   
        // Display all know activity templates
        RBActivityTemplateRecs recs = rbServer.getTemplates("", "");
        RBActivityTemplate templates[] = recs.subList(0, recs.size());
        for (int i=0; i<templates.length; i++) {
            RBActivityTemplate current = templates[i];
            if (current.isOnServer()) {
                System.out.println("template["+i+"]"
                        +" (on server)"
                        +" desc:"+current.getDescription()
                        +" scop:"+current.getScope()
                        +" endact:"+current.getEndAction());
            }
            else {
                System.out.println("template["+i+"]"
                        +" (incomplete)"
                        +" desc:"+current.getDescription()
                        +" scop:"+current.getScope());
            }
        }
        System.out.println();
        
        // Offer choice to copy multiple activity templates
        try {
            BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
            Vector v  = new Vector();
            int choice;
            do {
                choice = -1;
                do {
                    System.out.println("Please enter the number for the object you wish to copy (-1 to exit, letter to skip).\n");
                    choice = Integer.parseInt(inReader.readLine().trim());
                } while ( choice < -1 || choice >= templates.length );
                if (choice > -1) {
                    v.add(templates[choice]);
                }
            } while (choice > -1);
            if (v.size() > 0) {
                RBActivityTemplate[] tmpl = new RBActivityTemplate[v.size()];
                for (int i=0; i<v.size(); i++) {
                    tmpl[i] = (RBActivityTemplate)v.elementAt(i);
                    System.out.println("Selected:"+tmpl[i].getDescription());
                }
                Date now = new Date();
                // Schedule the copy activity, a variant would be to build bigger lists of objects
                RBActivity newAct = rbServer.registerObjectCopyActivity("objcopy", false, tmpl, null, null, 
                        now, new Date(now.getTime()+3600000), new Date(now.getTime()+86400000),false);
                // Wait for completion (or failure)
                RBActivityTargetRecs newActTargetRecs = newAct.getActivityTargets(); // expect only one
                RBActivityTarget target = newActTargetRecs.elementAt(0);
                int count = 0;
                while(count < 100) {
                    count++;
                    System.out.println(target);
                    if (target.getStatus().equals("FAILED") || target.getStatus().equals("COMPLETED")) {
                        break;
                    }
                    Thread.sleep(4000);
                    target = newActTargetRecs.elementAt(0);
                }
            }
        } catch (NumberFormatException e) {
        }

        // Display all known software packages
        RBSoftwarePackageRecs srecs = rbServer.getSoftwarePackages("", "");
        RBSoftwarePackage softs[] = srecs.subList(0, srecs.size());
        for (int i=0; i<softs.length; i++) {
            RBSoftwarePackage current = softs[i];
            if (current.isOnServer()) {
                System.out.println("soft["+i+"]"
                        +" (on server)"
                        +" desc:"+current.getDescription()
                        +" scop:"+current.getScope()
                        +" type:"+current.getType()
                        +" size:"+current.getUniqueKB()
                        +" sharedsize:"+current.getSharedKB());
            }
            else {
                System.out.println("soft["+i+"]"
                        +" (incomplete)"
                        +" desc:"+current.getDescription()
                        +" scop:"+current.getScope());
            }
        }
        System.out.println();
        
        // Offer choice to copy multiple software packages
        try {
            BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
            Vector v  = new Vector();
            int choice;
            do {
                choice = -1;
                do {
                    System.out.println("Please enter the number for the object you wish to copy (-1 to exit, letter to skip).\n");
                    choice = Integer.parseInt(inReader.readLine().trim());
                } while ( choice < -1 || choice >= softs.length );
                if (choice > -1) {
                    v.add(softs[choice]);
                }
            } while (choice > -1);
            if (v.size() > 0) {
                RBSoftwarePackage[] sp = new RBSoftwarePackage[v.size()];
                for (int i=0; i<v.size(); i++) {
                    sp[i] = (RBSoftwarePackage)v.elementAt(i);
                    System.out.println("Selected:"+sp[i].getDescription());
                }
                Date now = new Date();
                // Schedule the copy activity, a variant would be to build bigger lists of objects
                RBActivity newAct = rbServer.registerObjectCopyActivity("objcopy", false, null, null, sp, 
                        now, new Date(now.getTime()+3600000), new Date(now.getTime()+86400000),false);
                // Wait for completion (or failure)
                RBActivityTargetRecs newActTargetRecs = newAct.getActivityTargets(); // expect only one
                RBActivityTarget target = newActTargetRecs.elementAt(0);
                int count = 0;
                while(count < 100) {
                    count++;
                    System.out.println(target);
                    if (target.getStatus().equals("FAILED") || target.getStatus().equals("COMPLETED")) {
                        break;
                    }
                    Thread.sleep(4000);
                    target = newActTargetRecs.elementAt(0);
                }
            }
        } catch (NumberFormatException e) {
        }

        // Display all known system profiles
        RBSystemProfileRecs precs = rbServer.getSystemProfiles("", "");
        RBSystemProfile profs[] = precs.subList(0, precs.size());
        for (int i=0; i<profs.length; i++) {
            RBSystemProfile current = profs[i];
            if (current.isOnServer()) {
                System.out.println("prof["+i+"]"
                        +" (on server)"
                        +" desc:"+current.getDescription()
                        +" scop:"+current.getScope()
                        +" size:"+current.getUniqueKB()
                        +" sharedsize:"+current.getSharedKB());
            }
            else {
                System.out.println("prof["+i+"]"
                        +" (incomplete)"
                        +" desc:"+current.getDescription()
                        +" scop:"+current.getScope());
            }
        }
        System.out.println();
        
        // Offer choice to copy multiple system profiles
        try {
            BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
            Vector v  = new Vector();
            int choice;
            do {
                choice = -1;
                do {
                    System.out.println("Please enter the number for the object you wish to copy (-1 to exit, letter to skip).\n");
                    choice = Integer.parseInt(inReader.readLine().trim());
                } while ( choice < -1 || choice >= profs.length );
                if (choice > -1) {
                    v.add(profs[choice]);
                }
            } while (choice > -1);
            if (v.size() > 0) {
                RBSystemProfile[] sp = new RBSystemProfile[v.size()];
                for (int i=0; i<v.size(); i++) {
                    sp[i] = (RBSystemProfile)v.elementAt(i);
                    System.out.println("Selected:"+sp[i].getDescription());
                }
                Date now = new Date();
                // Schedule the copy activity, a variant would be to build bigger lists of objects
                RBActivity newAct = rbServer.registerObjectCopyActivity("objcopy", false, null, sp, null, now, 
                        new Date(now.getTime()+3600000), new Date(now.getTime()+86400000), false);
                // Wait for completion (or failure)
                RBActivityTargetRecs newActTargetRecs = newAct.getActivityTargets(); // expect only one
                RBActivityTarget target = newActTargetRecs.elementAt(0);
                int count = 0;
                while(count < 100) {
                    count++;
                    System.out.println(target);
                    if (target.getStatus().equals("FAILED") || target.getStatus().equals("COMPLETED")) {
                        break;
                    }
                    Thread.sleep(4000);
                    target = newActTargetRecs.elementAt(0);
                }
            }
        } catch (NumberFormatException e) {
        }
        System.out.println();
        
        // Display all known configurations (copied with system profiles)
        RBConfigurationRecs crecs = rbServer.getConfigurations("", "");
        RBConfiguration confs[] = crecs.subList(0, crecs.size());
        for (int i=0; i<confs.length; i++) {
            RBConfiguration current = confs[i];
            if (current.isOnServer()) {
                System.out.println("conf["+i+"]"
                        +" (on server)"
                        +" desc:"+current.getDescription()
                        +" scop:"+current.getScope()
                        +" fixed_hostname:"+current.getFixedHostName());
            }
            else {
                System.out.println("conf["+i+"]"
                        +" (incomplete)"
                        +" desc:"+current.getDescription()
                        +" scop:"+current.getScope());
            }
        }
   
        System.out.println("\nDone.");
    }
}
