/* TestLinuxPreboot.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestLinuxPreboot.java#9 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBPrebootExecuteTemplate;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.RBSoftwarePackage;
import com.ibm.rbapi.RBSoftwarePackageRecs;
import com.ibm.rbapi.exception.RBTimeoutException;

public class TestLinuxPreboot {

    public static void main(String[] args) 
    throws UnknownHostException, IOException, InterruptedException, 
    RBServerException, RBObjectNotFoundException, RBTimeoutException
    {
        
        // If the test program is called with 4 arguments consider that args[3] is
        // the archive filename in files/global/updates (deprecated).
        // If there are only three arguments, get the list of Linux images from the
        // server and let the user choose one.
        if (args.length < 3 || args.length > 4) {
            System.out.println("usage: TestLinuxPreboot <ip> <port> <password> [<linuximagefile>]\n");
            System.out.println("  where linuximagefile is the archive name in files/global/updatesn");
            return;
        }

        System.out.println("TestLinuxPreboot starting. We will test booting a Linux image on a host machine.");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret); 
        // Abort if the server is not ready after 15 seconds
        rbServer.isReady(15000);
        
        int ramDiskSize = 300*1024; // 300 MB
        HashMap paramMap = new HashMap();
        paramMap.put("kernelArgs", "");
        
        
        RBPrebootExecuteTemplate prebootExTempl;
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        if (args.length == 4) {
            String imageName = args[3];
            prebootExTempl = rbServer.registerPrebootExecuteTemplate("preboot Desc",
                    RBPrebootExecuteTemplate.OSTYPE_LINUX, RBPrebootExecuteTemplate.FSTYPE_EXT2, 
                    imageName, false, ramDiskSize, null, null, paramMap);
            System.out.println("\nCreated new PrebootExecuteTemplate for Linux image: " 
                    + imageName + ", description: " + prebootExTempl.getDescription());
        } else {
            System.out.println("\nRetrieving list of available Linux images...");
            RBSoftwarePackageRecs recs = rbServer.getSoftwarePackages(
                    "[Type]='"+RBSoftwarePackage.TYPE_RAMD+"' and [Content]='"
                    +RBSoftwarePackage.CONTENT_LINPE+"'", "");
            if (recs.size() == 0) {
                System.out.println("No Linux image found on the server, exiting.");
                return;
            }
            RBSoftwarePackage[] softs = recs.subList(0, recs.size()); 
            for (int i=0; i<softs.length; i++) {
                RBSoftwarePackage current = softs[i];
                System.out.println(i + ") " + current.getDescription() 
                        + " pkg: " + current.getSourceFileName());
            }
            int answer = -1;
            do {
                System.out.println("Please enter the number for the image you want to use: \n");
                answer = Integer.parseInt(inReader.readLine().trim());
            } while ( answer < 0 || answer >= recs.size() );
            RBSoftwarePackage choice = softs[answer];
            prebootExTempl = rbServer.registerPrebootExecuteTemplate("preboot Desc",
                    choice, RBPrebootExecuteTemplate.FSTYPE_EXT2, 
                    false, ramDiskSize, null, null, paramMap);
            System.out.println("\nCreated new PrebootExecuteTemplate for Linux image: " 
                    + choice.getSourceFileName() + ", description: " + choice.getDescription());
        }
        RBHostRecs hostRecs = null;
        while (true) {
            System.out.println("\nRetrieving list of available hosts...");
            // Now get list of hosts we can run it on
            hostRecs = rbServer.getHosts("", "");
            
            if (hostRecs.size() == 0) {
                System.out.println("\n\nThere are no hosts found!  This test script needs at least one host to run properly.");
                System.out.println("Please start a host so it boots off the TPMfOSd server and gets registered in the database." +
                        "\nPress enter to continue.");
                inReader.readLine();
            } else {
                break;
            }
        }
        
        RBHost[] availHosts = hostRecs.subList(0, hostRecs.size()); 
        for (int i=0; i<availHosts.length; i++) {
            System.out.println(i + ") " + availHosts[i].getIPAddress()
                    + " MAC: " + availHosts[i].getMacAddress()
                    + " Description: " + availHosts[i].getDescription());
        }
        
        int hostIndex = -1;
        do {
            System.out.println("Please enter the number for the host you wish to preboot with the Linux image: \n");
            inReader = new BufferedReader(new InputStreamReader(System.in));
            hostIndex = Integer.parseInt(inReader.readLine().trim());
        } while ( hostIndex < 0 || hostIndex >= hostRecs.size() );
        
        RBHost rbhost = hostRecs.elementAt(hostIndex);
        
        System.out.println("\n\nNow we will schedule the previously created " +
                "ActivityTemplate to run on the host with IP: " 
                + rbhost.getIPAddress().getHostAddress()
                + " and Description: " + rbhost.getDescription());
        
        /* Schedule Activity to use the PrebootExecuteTemplate we just created on user specified host */
        RBActivity newAct = rbServer.registerActivity("Test Activity using Linux image", 
                prebootExTempl, false, false, null, rbhost);
        System.out.println("\nCreated new activity that uses the RBPrebootExecuteTemplate " +
                "we just created to run immediately on host with IP: " 
                + rbhost.getIPAddress().getHostAddress());
        printActivity(newAct); 
        
        
        System.out.println("\nPress Enter when you are done testing the Linux preboot execution\n" +
                " and I will remove the template to clean up:");
        inReader.readLine();
        
        /* Now remove the template 
         * 
         * We are just doing this to cleanup - normally if you know you 
         * will run a linux preboot activity many times, you can just create
         * the template once and not remove it.
         */
        System.out.println("\nRemoving the previously created template...");
        rbServer.removeTemplate(prebootExTempl, true);
        System.out.println("Done.");
    }
    
    
    public static void printMap(Map map) {
        if (map != null) {
            Iterator keyIterator = (map.keySet()).iterator();
            while (keyIterator.hasNext()) {
                String key = (String) keyIterator.next();
                System.out.println("## key: " + key + " value: " + (map.get(key)).toString());
            }
        }
    }
    
    public static void printActivity(RBActivity activity) {
        System.out.println("\n### RBActivity - printing out ###");
        System.out.println("###  getDescription(): " + activity.getDescription());
        System.out.println("###  getDoneDate(): " + activity.getDoneDate());
        System.out.println("###  getExecDate(): " + activity.getExecDate());
        System.out.println("###  getExpireDate(): " + activity.getExpireDate());
        System.out.println("###  getProgress(): " + activity.getProgress());
        System.out.println("###  isSuspendOnError(): " + activity.isSuspendOnError());
    }
    
    public static void printActivityTarget(RBActivityTarget target) {
        System.out.println("\n### RBActivityTarget - printing out ###");
        System.out.println("###  getProgress(): " + target.getProgress());
        System.out.println("###  getStatus(): " + target.getStatus());
    }
}
