/* TestRADExport.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestRADExport.java#1 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Date;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBActivityTargetResults;
import com.ibm.rbapi.RBActivityTemplate;
import com.ibm.rbapi.RBActivityTemplateRecs;
import com.ibm.rbapi.RBEvent;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.RBSoftwarePackage;
import com.ibm.rbapi.RBSoftwarePackageRecs;
import com.ibm.rbapi.RBSystemProfile;
import com.ibm.rbapi.RBSystemProfileRecs;
import com.ibm.rbapi.RBWebExtensionStatus;

/*
 * Example shows exporting objects to a RAD file.  The target host for this 
 * activity must be a host running the Web Interface Extension (WIE).
 */
public class TestRADExport {

    public static void main(String[] args) 
    throws UnknownHostException, IOException, InterruptedException, 
    RBServerException, RBObjectNotFoundException 
    {
        if (args.length < 3) {
            System.out.println("usage: TestRADExport <ip> <port> <password>");
            return;
        }

        System.out.println("TestRADExport starting.");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret); 
        System.out.println("Created rbServer.\n");
        
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        RBHostRecs hostRecs = rbServer.getHosts("", "");
        RBHost[] availHosts = hostRecs.subList(0, hostRecs.size()); 
        for (int i=0; i<availHosts.length; i++) {
            System.out.println(i + ") " + availHosts[i].getIPAddress()
                    + " MAC: " + availHosts[i].getMacAddress()
                    + " Description: " + availHosts[i].getDescription());
        }
        
        int hostIndex = -1;
        do {
            System.out.println("Please enter the number for the host running the WIE you wish to use to export to a rad file: \n");
            hostIndex = Integer.parseInt(inReader.readLine().trim());
        } while ( hostIndex < 0 || hostIndex >= hostRecs.size() );
        
        RBHost rbhost = availHosts[hostIndex];
        
        // Check if Web Extension is ready on that host
        System.out.println("Checking RBServer.isWebExtensionReady() for host: " + rbhost.getIPAddress() + "...");
        RBWebExtensionStatus wieStatus = null;
        for (int i=0; i<10; i++) {  // try at most 10 times (20 seconds)
            wieStatus = rbServer.isWebExtensionReady(rbhost.getIPAddress());
            if (wieStatus.isPresent()) {
                break;
            }
            Thread.sleep(2000);
        }
        System.out.println("WIE - isPresent(): " + wieStatus.isPresent());
        System.out.println("WIE - getWebExtensionVersion(): " + wieStatus.getWebExtensionVersion());
        System.out.println("WIE - getWebExtensionBuild(): " + wieStatus.getWebExtensionBuild());
        System.out.println("WIE - getServerVersion(): " + wieStatus.getServerVersion());
        System.out.println("WIE - getServerBuild(): " + wieStatus.getServerBuild());
        
        if (!wieStatus.isPresent()) {
            System.out.println("The Web Interface Extension is not detected on the target host.  " +
                    "Please ensure that it is started and run this test again.");
            System.exit(1);
        }
        
        String choice = "";
        while (!choice.equals("1") && !choice.equals("2")) {
            System.out.println("\n\nPlease enter a number to choose one of the following: \n");
            System.out.println("1) Export System Profile(s).");
            System.out.println("2) Do not export system profiles.");
            choice = inReader.readLine().trim(); 
        }
        RBSystemProfile[] profilesToExport = null;
        ArrayList sysProfileArray = new ArrayList();
        if (choice.equals("1")) {
            System.out.println("Getting list of system profiles from server...");
            RBSystemProfileRecs sysProfRecs = rbServer.getSystemProfiles("", "");
            RBSystemProfile[] allProfiles = sysProfRecs.subList(0, sysProfRecs.size());
            
            System.out.println("\nEnter the number of the system profile you wish to export:");
            int userInput = -1;
            while (userInput != allProfiles.length) {
                for (int i=0; i<allProfiles.length; i++) {
                    System.out.println(i+") " + allProfiles[i].getDescription());
                }
                System.out.println(allProfiles.length+") Done, continue.");
                userInput = Integer.parseInt(inReader.readLine().trim());
                
                if (userInput >= 0 && userInput < allProfiles.length) {
                    sysProfileArray.add(allProfiles[userInput]);
                }
            }
            profilesToExport = new RBSystemProfile[sysProfileArray.size()];
            for (int j=0; j<sysProfileArray.size(); j++) {
                profilesToExport[j] = (RBSystemProfile) sysProfileArray.get(j);
            }
        }
        
        choice = "";
        while (!choice.equals("1") && !choice.equals("2")) {
            System.out.println("\n\nPlease enter a number to choose one of the following: \n");
            System.out.println("1) Export Software Package(s).");
            System.out.println("2) Do not export software packages.");
            choice = inReader.readLine().trim(); 
        }
        RBSoftwarePackage[] softwareToExport = null;
        ArrayList softwareArray = new ArrayList();
        if (choice.equals("1")) {
            System.out.println("Getting list of software packages from server...");
            RBSoftwarePackageRecs softPkgRecs = rbServer.getSoftwarePackages("", "");
            RBSoftwarePackage[] allSoftwares = softPkgRecs.subList(0, softPkgRecs.size());
            
            System.out.println("\nEnter the number of the Software package you wish to export:");
            int userInput = -1;
            while (userInput != allSoftwares.length) {
                for (int i=0; i<allSoftwares.length; i++) {
                    System.out.println(i+") " + allSoftwares[i].getDescription());
                }
                System.out.println(allSoftwares.length+") Done, continue.");
                userInput = Integer.parseInt(inReader.readLine().trim());
                
                if (userInput >= 0 && userInput < allSoftwares.length) {
                    softwareArray.add(allSoftwares[userInput]);
                }
            }
            softwareToExport = new RBSoftwarePackage[softwareArray.size()];
            for (int j=0; j<softwareArray.size(); j++) {
                softwareToExport[j] = (RBSoftwarePackage) softwareArray.get(j);
            }
        }
        
        choice = "";
        while (!choice.equals("1") && !choice.equals("2")) {
            System.out.println("\n\nPlease enter a number to choose one of the following: \n");
            System.out.println("1) Export Activity Template(s).");
            System.out.println("2) Do not export activity templates.");
            choice = inReader.readLine().trim(); 
        }
        RBActivityTemplate[] templatesToExport = null;
        ArrayList templateArray = new ArrayList();
        if (choice.equals("1")) {
            System.out.println("Getting list of templates from server...");
            RBActivityTemplateRecs templRecs = rbServer.getTemplates("", "");
            RBActivityTemplate[] allTemplates = templRecs.subList(0, templRecs.size());
            
            System.out.println("\nEnter the number of the Template you wish to export:");
            int userInput = -1;
            while (userInput != allTemplates.length) {
                for (int i=0; i<allTemplates.length; i++) {
                    System.out.println(i+") " + allTemplates[i].getDescription());
                }
                System.out.println(allTemplates.length+") Done, continue.");
                userInput = Integer.parseInt(inReader.readLine().trim());
                
                if (userInput >= 0 && userInput < allTemplates.length) {
                    templateArray.add(allTemplates[userInput]);
                }
            }
            templatesToExport = new RBActivityTemplate[templateArray.size()];
            for (int j=0; j<templateArray.size(); j++) {
                templatesToExport[j] = (RBActivityTemplate) templateArray.get(j);
            }
        }
        
        System.out.println("\n\nPlease enter the filename of the rad file to be" +
                " created.  This should be the full path to the file to be created on the host: ");
        String radFile = inReader.readLine().trim();
        
        
        System.out.println("\n\nWe will now export to file " + radFile + "...");
        
        Date now = new Date();
        Date stop = new Date(now.getTime()+1000*3600);
        Date expire = new Date(now.getTime()+1000*3600*24);
        RBActivity act = rbServer.registerRADExportActivity("TestRADExportActivity", 
                false, null, rbhost, radFile, profilesToExport, 
                softwareToExport, templatesToExport, now, stop, expire);
        
        /*
         * This section here refreshes the activity target to see if the activity
         * is complete or not.  This is done just to code something quickly, but 
         * is not very efficient as the polling results in making queries to the 
         * server quite often.  For an example of better activity event monitoring,
         * see the TestRBEvents.java example class, or the TestStressSmart.java class.
         */
        Date lastTime = new Date();
        boolean foundCompletedEvent = false;
        while (!foundCompletedEvent) {
            Thread.sleep(8000);
            System.out.println("Getting all events for this activity from server from time: " + lastTime);
            RBEvent[] events =  act.getEventsFromServer(lastTime);
            RBEvent e = null;
            for (int i=0; i<events.length; i++) {
                e = events[i];

                System.out.println("Event: Progress=" + e.getProgress() + 
                        " Progress Msg=" + e.getLastProgressMsg() + 
                        " Status=" + e.getStatus());
                
                // Let's get information about events that are completed or failed
                if ( e.getStatus().equals(RBActivityTarget.COMPLETED) ||
                        e.getStatus().equals(RBActivityTarget.FAILED)) {
                    System.out.println("Getting corresponding RBActivityTarget object");
                    RBActivityTarget ourActTarget = e.getRBActivityTarget();
                    System.out.println("Activity Target: Progress=" + ourActTarget.getProgress() + 
                            " Progress Msg=" + ourActTarget.getLastProgressMsg() + 
                            " Status=" + ourActTarget.getStatus());
                    
                    // Print out the specific results for the activity target
                    RBActivityTargetResults results = ourActTarget.getActivityTargetResults();
                    System.out.println("### Results for our software creation activity: ");
                    System.out.println("Status: " + results.getStatus());

                    foundCompletedEvent = true;
                    System.out.println("Exiting loop.");
                }
            }
            if (e != null) {
                lastTime = e.getUpdateTime();
            } else {
                System.out.println("No events found.");
            }
        }
        
        System.out.print("\nTestRADExport done.");
    }
}
