/* TestRBEvents.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2006
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestRBEvents.java#8 $
 */
package examples;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBEvent;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;

/*
 * This test program will constantly poll the server for new events every 4 
 * seconds.  If a new event is found that is COMPLETED or FAILED, it will
 * exit.  If you want the program to keep running, then you can pass in true
 * as the 4th parameter, "loopForever" and it will not exit after finding a
 * COMPLETED or FAILED activity.
 */
public class TestRBEvents {

    public static void main(String[] args) 
    throws UnknownHostException, InterruptedException, RBServerException, RBObjectNotFoundException {
        
        if (args.length < 3) {
            System.out.println("usage: TestEvents <ip> <port> <password> <loopForever(true|false)>");
            return;
        }

        System.out.println("TestEvents starting.  For this test program, we will simply loop and keep querying " +
                "for new events until an event is found that is COMPLETED or FAILED.  Then we will " +
                "get some more information about that event and exit.");
        System.out.println("\nThe easiest thing to do is to launch this program, then run another which " +
                "initiates an activity on a target.");
        System.out.println("\nNote that if you remove your template at the end of a program as many of the " +
                "test programs do, it will remove the corresponding events, in which case the COMPLETED event may " +
                "not be caught by this program.");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String token = args[2];
        boolean loopForever = false;
        if (args.length >= 4) {
            loopForever = new Boolean(args[3]).booleanValue();
            System.out.println("We will loop forever and monitor events");
        }
        
        RBServer rbServer = new RBServer(ip, port, token); 
        System.out.println("Created rbServer.\n");
        
        Date lastTime = new Date();
        boolean foundCompletedEvent = false;
        
        while (!foundCompletedEvent) {
            Thread.sleep(400);
            System.out.println("Getting all events from server from time: " + lastTime);
            RBEvent[] events = rbServer.getEvents(lastTime);
            
            RBEvent e = null;
            for (int i=0; i<events.length; i++) {
                e = events[i];
                
                System.out.println("Event: Progress=" + e.getProgress() + " Status=" + e.getStatus());
                
                // Let's get information about events that are completed or failed
                if ( e.getStatus().equals(RBActivityTarget.COMPLETED) ||
                        e.getStatus().equals(RBActivityTarget.FAILED)) {
                    System.out.println("Event status is: " + e.getStatus() + " Now let's get some more info from the server\n"); 
                    
                    // Different ways to get information, here are some examples (you would probably not use these all at the same time):
                    System.out.println("Getting corresponding RBHost object");
                    RBHost rbHost = e.getRBHost();
                    System.out.println(" The event that just completed was run on host: " 
                            + rbHost.getDescription() 
                            + ", " + rbHost.getIPAddress()
                            + ", " + rbHost.getMacAddress());
                    
                    System.out.println("Getting corresponding RBActivity object");
                    RBActivity rbAct = e.getRBActivity();
                    System.out.println(" The event that just completed was part of Activity: " + rbAct.getDescription());
                    
                    System.out.println("Getting corresponding RBActivityTarget object");
                    RBActivityTarget rbActTarget = e.getRBActivityTarget();
                    System.out.println("Results: params -- ");
                    printMap(rbActTarget.getParams());
                    System.out.println("Results: errors -- ");
                    printMap(rbActTarget.getError());
                    
                    // Also note that if you already have instances of an RBActivity, RBActivityTarget or RBHost, you can use the RBEvent.belongsTo* methods
                    // to see if they belong to your already instantiated objects, rather than going to the server to retrieve new ones, as in the 
                    // code above.
                    
                    if (!loopForever) {
                        foundCompletedEvent = true;
                        System.out.println("Exiting loop.");
                    }
                }
            }
            if (e != null) {
                lastTime = e.getUpdateTime();
            } else {
                System.out.println("No events found.");
            }
        }
    }
    
    public static void printMap(Map map) {
        if (map != null) {
            Iterator keyIterator = (map.keySet()).iterator();
            while (keyIterator.hasNext()) {
                String key = (String) keyIterator.next();
                System.out.println("## key: " + key + " value: " + (map.get(key)).toString());
            }
        }
    }
} 
