/* TestRBImageCaptureTemplate.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestRBImageCaptureTemplate.java#10 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBImageCaptureTemplate;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;

public class TestRBImageCaptureTemplate {

    public static void main(String[] args) 
    throws UnknownHostException, IOException, InterruptedException, 
    RBServerException, RBObjectNotFoundException 
    {
        
        if (args.length < 3) {
            System.out.println("usage: TestRBImageCaptureTemplate <ip> <port> <password> \n");
            return;
        }

        System.out.println("TestRBImageCaptureTemplate starting.  We will test capturing an OS image from a host machine");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret); 
        
        RBImageCaptureTemplate imageTempl = rbServer.registerImageCaptureTemplate("CaptureImageTestTemplate", false, false, false, true);
        System.out.println("\nCreated new ImageCaptureTemplate with description: " + imageTempl.getDescription());
        
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        RBHostRecs hostRecs = null;
        while (true) {
            System.out.println("\nRetrieving list of available hosts...");
            // Now get list of hosts we can run it on
            hostRecs = rbServer.getHosts("", "");
            
            if (hostRecs.size() == 0) {
                System.out.println("\n\nThere are no hosts found!  This test class needs at least one host to run properly.");
                System.out.println("Please start a host so it boots off the TPMfOSd server and gets registered in the database." +
                        "\nPress enter to continue.");
                inReader.readLine();
            } else {
                break;
            }
        }
        
        RBHost[] availHosts = hostRecs.subList(0, hostRecs.size()); 
        for (int i=0; i<availHosts.length; i++) {
            System.out.println(i + ") " + availHosts[i].getIPAddress()
                    + " MAC: " + availHosts[i].getMacAddress()
                    + " Description: " + availHosts[i].getDescription());
        }
        
        int hostIndex = -1;
        do {
            System.out.println("Please enter the number for the host you wish to capture the image of: \n");
            inReader = new BufferedReader(new InputStreamReader(System.in));
            hostIndex = Integer.parseInt(inReader.readLine().trim());
        } while ( hostIndex < 0 || hostIndex >= hostRecs.size() );
        
        RBHost rbhost = hostRecs.elementAt(hostIndex);
        
        System.out.println("\n\nNow we will schedule the previously created " +
                "ImageCaptureTemplate to run on the host with IP: " 
                + rbhost.getIPAddress()
                + " and Description: " + rbhost.getDescription());
        
        
        /* Schedule Activity to use the ImageCaptureTemplate we just created on user specified host */
        /* Note here - you must supply two parameters 1) "profiledesc" which is the description of the profile to create.
         *                                            2) "profilecomment" which is the comment for the profile.
         */
        HashMap paramMap = new HashMap();
        paramMap.put("profiledesc", "TestProfileFromImageCapture" + hostIndex);
        paramMap.put("profilecomment", "Comment goes here.");
        paramMap.put("DiskToCapture", "all");
        RBActivity newAct = rbServer.registerActivity("Test Image Capture Activity ", 
                imageTempl, false, false, paramMap, rbhost);
        System.out.println("\nCreated new activity to capture the image immediately " +
                "on host with IP: " + rbhost.getIPAddress());
        printActivity(newAct); 
        
       
        System.out.println("\nPress Enter when you are done testing the image capture \n" +
        " and I will remove the template to clean up:");
        inReader.readLine();
        
        /*
         * While the image capture is running, you can monitor the progress
         * with the TestRBEvents class for example.
         */

        /* Now remove the template 
         * 
         * We are just doing this to cleanup - normally if you know you 
         * will run a linux preboot activity many times, you can just create
         * the template once and not remove it.
         */
        System.out.println("Removing the previously created template...");
        rbServer.removeTemplate(imageTempl, true);
        System.out.println("Done.");
    }
    
    public static void printMap(Map map) {
        if (map != null) {
            Iterator keyIterator = (map.keySet()).iterator();
            while (keyIterator.hasNext()) {
                String key = (String) keyIterator.next();
                System.out.println("## key: " + key + " value: " + (map.get(key)).toString());
            }
        }
    }
    
    public static void printActivity(RBActivity activity) {
        System.out.println("\n### RBActivity - printing out ###");
        System.out.println("###  getDescription(): " + activity.getDescription());
        System.out.println("###  getDoneDate(): " + activity.getDoneDate());
        System.out.println("###  getExecDate(): " + activity.getExecDate());
        System.out.println("###  getExpireDate(): " + activity.getExpireDate());
        System.out.println("###  getProgress(): " + activity.getProgress());
        System.out.println("###  isSuspendOnError(): " + activity.isSuspendOnError());
    }
    
    public static void printActivityTarget(RBActivityTarget target) {
        System.out.println("\n### RBActivityTarget - printing out ###");
        System.out.println("###  getProgress(): " + target.getProgress());
        System.out.println("###  getStatus(): " + target.getStatus());
    }
}
