/* TestRBOSDeployment.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestRBOSDeployment.java#22 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.HashMap;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTemplate;
import com.ibm.rbapi.RBActivityTemplateRecs;
import com.ibm.rbapi.RBConfiguration;
import com.ibm.rbapi.RBConfigurationRecs;
import com.ibm.rbapi.RBDeploymentParams;
import com.ibm.rbapi.RBDiskLayout;
import com.ibm.rbapi.RBDiskPartition;
import com.ibm.rbapi.RBDiskPartitionMode;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBOSDeploymentTemplate;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.RBSoftwarePackage;
import com.ibm.rbapi.RBSoftwarePackageRecs;
import com.ibm.rbapi.exception.RBTimeoutException;

public class TestRBOSDeployment {

    private final static String TEMPLATE_DESC = "deplscheme-java";

    /*
     * Sample code for RBOSDeploymentTemplate
     * At least one configuration must exist on your TPMFOSD server
     * At least one host must be registered in TPMFOSD database
     * If optional 4th argument "cleanup" is present, remove previous tests
     * A new RBOSDeploymentTemplate is created
     * An activity is started
     */
    public static void main(String[] args) 
    throws UnknownHostException, RBServerException, RBObjectNotFoundException,
    IOException, RBTimeoutException
    {

        if (args.length < 3) {
            System.out.println(
                    "usage: TestRBOSDeployment <ip> <port> <password> [cleanup]");
            return;
        }

        System.out.println("TestRBOSDeployment starting.");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret);
        System.out.println("Created rbServer.\n");
        // Abort if the server is not ready after 15 seconds
        rbServer.isReady(15000);

        /*
         * If requested, remove all objects created by previous tests
         * NOTE: for the moment, only the database is cleaned, not the server 
         * filesystem
         * NOTE: the matching expression in getTemplates must be refined
         */
        if (args.length==4 && args[3].equals("cleanup")) {
            RBActivityTemplateRecs recs = rbServer.getTemplates("[Type]='DEPLOY'", "");
            for (int i=0; i<recs.size(); i++) {
                RBActivityTemplate current = recs.elementAt(i);
                System.out.println("Removing template "+ 
                        current.getDescription());
                rbServer.removeTemplate(current, true);
            }
            System.out.println();
        }
        
        /*
         * Create a new RBOSDeploymentTemplate with no configuration specified
         */
        RBOSDeploymentTemplate depl = 
            rbServer.registerOSDeploymentTemplate(TEMPLATE_DESC, null);
        System.out.println(depl);
 
        /*
         * Update the template
         */
        depl.setUnicast(false);
        depl.setMCASTGroup("arbitrarygroupname");
        
        // uncomment following lines for customized deployment scheme
        // depl.setUserConfirm(true);
        // depl.setEditBOM("reallyalways");
        depl.setEditBOM("never");
        
        depl.update();
        System.out.println(depl);

        /*
         * Retreive and display all know configurations.
         * Remember the last one in variable "current".
         */
        RBConfigurationRecs confRecs = rbServer.getConfigurations("","");
        RBConfiguration[] availConfigs = confRecs.subList(0, confRecs.size()); 
        RBConfiguration current = null;
        for(int i=0; i<confRecs.size(); i++) {
            System.out.println(i + ") Configuration(\""
                    + availConfigs[i].getDescription() + "\"): ");
        }
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        int configIndex = -1;
        do {
            System.out.println("Please enter the number for the configuration you " +
                    "wish to use: \n");
            configIndex = Integer.parseInt(inReader.readLine().trim());
        } while ( configIndex < 0 || configIndex >= confRecs.size() );
        
        current = confRecs.elementAt(configIndex);
        
        
        /*
         * Use multicast & send changes to server
         */
        System.out.println(depl);
        depl.setUnicast(false);
        depl.setMCASTGroup("arbitrarygroupname");
        depl.update();


        /*
         * Retrieves the disk layout information for the configurations and 
         * modifies them.
         */
        RBConfiguration rbConfig = current;
        RBDiskLayout diskLayout = rbConfig.getDiskLayout();

        do {
	        System.out.println(diskLayout.toString());
			System.out.println("\n");
            System.out.println("Please enter the partition to modify (disk:partition), any letter to exit");
			String[] diskPart = inReader.readLine().split(":");
			if (diskPart.length != 2) {
				break;
			}

			try {
				int diskNumber = Integer.parseInt(diskPart[0]);
				int partNumber = Integer.parseInt(diskPart[1]);
				RBDiskPartition partition = diskLayout.getPartition(diskNumber, partNumber);

				boolean goback = false;
				do {
					System.out.println(partition.toString());
					System.out.println("\n");
					System.out.println("Select changes to the partition:");
					System.out.println("1 - Set Partition Size in MB");
					System.out.println("2 - Set Partition Size as percentage");
					System.out.println("3 - Modify Disk/Partition number");
					System.out.println("4 - Set Mode - Format, Synchronize");
					System.out.println("5 - Set Mode - Synchronize");
					System.out.println("6 - Set Mode - Synchronize, Keep Files");
					System.out.println("7 - Set Mode - Do not touch");
					System.out.println("Any other key to go back to select another partition to edit");

					int intPartSelection;
					String partSelection = inReader.readLine();
					try {
						intPartSelection = Integer.parseInt(partSelection);
					} catch (Throwable th) {
						// Invalid selection, go back
						break;
					}
					
					switch (intPartSelection) {
					case 1:
						System.out.println("Enter partition size in MB(min: " + partition.getMinPartitionSize() + ") > ");
						int size = Integer.parseInt(inReader.readLine());
						partition.setPartitionSize(size, false);
						break;
					case 2:
						System.out.println("Enter partition size in as percentage > ");
						size = Integer.parseInt(inReader.readLine());
						partition.setPartitionSize(size, true);
						break;
					case 3:
						System.out.println("Enter new disk_number:partition_number > ");
						String newDiskPart = inReader.readLine();
						String[] newDiskParts = newDiskPart.split(":");
						try {
							partition.setDiskNumber(Integer.parseInt(newDiskParts[0]));
							partition.setPartitionNumber(Integer.parseInt(newDiskParts[1]));
						} catch (Throwable th) {
							System.out.println("Disk/partition number invalid, not saved.");
						}
						break;
					case 4:
						partition.setDiskPartitionMode(RBDiskPartitionMode.FORMAT_SYNCHRONIZE);
						break;
					case 5:
						partition.setDiskPartitionMode(RBDiskPartitionMode.SYNCHRONIZE);
						break;
					case 6:
						partition.setDiskPartitionMode(RBDiskPartitionMode.SYNCHRONIZE_KEEP_FILES);
						break;
					case 7:
						partition.setDiskPartitionMode(RBDiskPartitionMode.DO_NOT_TOUCH);
						break;
					default:
						goback = true;
					}
					
				} while (!goback);
			} catch (Throwable th) {
				// Invalid selection, exit
				break;
			}
        } while ( true );
		

        // Print out the modifications to the disks layout for verification
        System.out.println("Modified disk layout: ");
        System.out.println(diskLayout.toString());
        
        
        /*
         * Install additional software packages 
         * 
         * This example is just done quickly to allow you to try out different
         * software package installs.
         * 
         * Usually you will specify software packages to install and you can
         * do this by doing getSoftwarePackages with a specific filter, then
         * using the subList method to get a RBSoftwarePackage array.  The 
         * example below is a little clumsy because we're just blindly adding
         * stuff to the list.
         */
        RBSoftwarePackageRecs softPkgs = rbServer.getSoftwarePackages("", "");
        String currentSoftSeq = "";
        ArrayList softSeqList = new ArrayList();
        ArrayList softInstallList = new ArrayList();
        if (softPkgs.size() > 0) {
            RBSoftwarePackage[] allSoft = softPkgs.subList(0, softPkgs.size());
            int softInput = -1;
            while (softInput < 0 || softInput != softPkgs.size()+1) {
                System.out.println("If you want to set software to install, enter " +
                        "the number of the software to install:");
                System.out.println("Current software install sequence is: " + currentSoftSeq);
                for (int i=0; i<softPkgs.size(); i++) {
                    System.out.println(i+") " + allSoft[i].getDescription());
                }
                System.out.println(softPkgs.size()+") Insert reboot action");
                System.out.println((softPkgs.size()+1)+") Done.");
                
                softInput = Integer.parseInt(inReader.readLine());
                if (softInput == softPkgs.size()) {
                    softSeqList.add(RBDeploymentParams.SEQUENCE_REBOOT);
                    currentSoftSeq = currentSoftSeq + "Reboot, ";
                } else if (softInput >= 0 && softInput < softPkgs.size() ) {
                    softSeqList.add(allSoft[softInput]);
                    softInstallList.add(allSoft[softInput]);
                    currentSoftSeq = currentSoftSeq + "Install " 
                        + allSoft[softInput].getDescription() + ", ";
                }
            }
        }
        RBSoftwarePackage[] softsToInstall = new RBSoftwarePackage[softInstallList.size()];
        for (int i=0; i<softInstallList.size(); i++) {
            softsToInstall[i] = (RBSoftwarePackage) softInstallList.get(i);
        }
      
        
        /* 
         * Use the new template for a deployment.
         * Ask the server about all possible targets and choose the next one! 
         */
        RBHostRecs hostRecs = rbServer.getHosts("", "");
        RBHost[] availHosts = hostRecs.subList(0, hostRecs.size()); 
        for (int i=0; i<availHosts.length; i++) {
            System.out.println(i + ") " + availHosts[i].getIPAddress()
                    + " MAC: " + availHosts[i].getMacAddress() 
                    + " Description: " + availHosts[i].getDescription());
        }
        
        int hostIndex = -1;
        do {
            System.out.println("Please enter the number for the host you wish to deploy to: \n");
            inReader = new BufferedReader(new InputStreamReader(System.in));
            hostIndex = Integer.parseInt(inReader.readLine().trim());
        } while ( hostIndex < 0 || hostIndex >= hostRecs.size() );
        
        RBHost rbhost = availHosts[hostIndex];
        
        System.out.println("\nWhen deploying the configuration, we will specifically" +
                " set the hostname.  Please enter the hostname you wish to use: \n");
        String hostname = inReader.readLine().trim();
        
        System.out.println("\nWe will also create a new user on the system.  Please \n" +
                "enter the name of the user to be created: \n");
        String username = inReader.readLine().trim();
        
        System.out.println("\nAsking server to deploy "+current.getDescription() + 
                "\non target "+rbhost.getMacAddress() + 
                "\nusing configuration: "+current.getDescription() +
                "\nwill set hostname to: " + hostname +
                "\nwill create user account: " + username);


        /* When registering the activity for an RBOSDeployment template, you 
         * can pass in a param in the param map called "RBDeploymentParams"
         * which is an RBDeploymentParams object.  This will be parameters
         * that will override the defaults from the template.  You can also 
         * pass in the param called "Configuration" to specify a specific
         * configuration to be used for the activity.
         */
        HashMap paramMap = new HashMap();
        RBDeploymentParams deplParams = new RBDeploymentParams();  
        deplParams.setHostName(hostname);   //Set the hostname to the specified hostname
        /* Now we will also use the customized disk layout that we modified earlier */
        deplParams.setDiskLayout(diskLayout);
        /* Now let's add our software list and our software sequence to the RBDeploymentParams */
        deplParams.setSoftwareToDeploy(softsToInstall);
        deplParams.setSoftwareToDeploySequence(softSeqList);
        
        /* Create a local user account, and set the name to the one entered by the user */
        deplParams.setCreateLocalAccount(true);
        deplParams.setUserName(username);
        
        paramMap.put("Configuration", current);
        /* Set values to be used on the host - these will override
         * values from the configuration and from the RBHost it's deployed
         * to as well.
         */
        paramMap.put("RBDeploymentParams", deplParams);
        RBActivity act = rbServer.registerActivity("TestActivity for OSDeployment", 
                depl, false, true, paramMap, rbhost);
        
        
        /*
         * Here is an example of adding another target to a task.
         * 
         * Notice as with the registerActivity, that when we're dealing
         * with an RBOSDeploymentTemplate, that we can add a RBDeploymentParams
         * object to the parameterMap to override RBDeploymentParams for this
         * particular target.
         */
        // Let's add another target to the task
 /*       int anotherHostIndex = hostIndex + 1;
        if (anotherHostIndex < hostRecs.size()) {
            RBHost anotherHost1 = hostRecs.elementAt(anotherHostIndex);
            System.out.println("\nAdding the host: " 
                    + anotherHost1.getDescription() + ", " + anotherHost1.getIP() 
                    + " to the deployment activity - but this time setting " +
                            "hostname to: TESTHOST1");
            RBDeploymentParams anotherDeploymentParams = new RBDeploymentParams();
            anotherDeploymentParams.setHostName("TESTHOST1");
            HashMap anotherPMap = new HashMap();
            anotherPMap.put("RBDeploymentParams", anotherDeploymentParams);
            act.addTarget(anotherHost1, anotherPMap);
        } */
        
        /*
         * At this stage, the deployment is scheduled to start on the PXE client.
         */
        System.out.println("Finished. Watch deployment on PXE client.");
        
        System.out.println("\nPress enter when the deployment is finished and I \n" +
                "will remove the template to clean up.");
        inReader.readLine();
        System.out.println("Removing template...");
        rbServer.removeTemplate(depl, true);
        System.out.println("\nDone.");
    }
}
