/* TestRBOSRestoreTemplate.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestRBOSRestoreTemplate.java#9 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.logging.Level;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBOSRestoreTemplate;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.RBSystemProfile;
import com.ibm.rbapi.RBSystemProfileRecs;

public class TestRBOSRestoreTemplate {

    public static void main(String[] args) 
    throws UnknownHostException, IOException, InterruptedException, 
    RBServerException, RBObjectNotFoundException 
    {
        
        if (args.length < 3) {
            System.out.println("usage: TestRBOSRestoreTemplate <ip> <port> <password>\n");
            return;
        }

        System.out.println("TestRBOSRestoreTemplate starting.  We will test restoring a systemprofile on a host machine");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret); 
        
        RBOSRestoreTemplate restoreTempl = rbServer.registerOSRestoreTemplate("test osrestore", false, false, false);
        System.out.println("\nCreated new OSRestoreTemplate, description: " + restoreTempl.getDescription());
        
        /* Optionally set some parameters for the template */
        restoreTempl.setUnicast(false);
        restoreTempl.setMCASTGroup("myMCASTgroup");
        restoreTempl.setNetEncrypt(true);
        restoreTempl.setSynchroHosts(2);
        restoreTempl.setSynchroDelay(20);
        restoreTempl.update();
        
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        RBHostRecs hostRecs = null;
        while (true) {
            System.out.println("\nRetrieving list of available hosts...");
            // Let's get a listing of all hosts
            hostRecs = rbServer.getHosts("", "");
            
            if (hostRecs.size() == 0) {
                System.out.println("\n\nThere are no hosts found!  This test script needs at least one host to run properly.");
                System.out.println("Please start a host so it boots off the TPMfOSd server and gets registered in the database." +
                        "\nPress enter to continue.");
                inReader.readLine();
            } else {
                break;
            }
        }
        
        RBHost[] availHosts = hostRecs.subList(0, hostRecs.size()); 
        for (int i=0; i<availHosts.length; i++) {
            System.out.println(i + ") " + availHosts[i].getIPAddress()
                    + " MAC: " + availHosts[i].getMacAddress()
                    + " Description: " + availHosts[i].getDescription());
        }
        
        int hostIndex = -1;
        do {
            System.out.println("Please enter the number for the host you wish to restore an image on: \n");
            System.out.flush();
            hostIndex = Integer.parseInt(inReader.readLine().trim());
        } while ( hostIndex < 0 || hostIndex >= hostRecs.size() );
        RBHost rbhost = availHosts[hostIndex];
        
        
        System.out.println("Looking for SystemProfiles to use..."); 
        RBSystemProfileRecs profRecs = rbServer.getSystemProfiles("", "");
        if (profRecs.size() == 0) {
            System.out.println("\nNo SystemProfiles found.  Please create a SystemProfile and re-run this test.");
            System.exit(1);
        }
        System.out.println("Found: " + profRecs.size() + " profiles!");;
        RBSystemProfile[] profiles = profRecs.subList(0, profRecs.size());
        for (int i=0; i<profiles.length; i++) {
            System.out.println(i + ") Description: " + profiles[i].getDescription() 
                    + ", comment: " + profiles[i].getComment());
        }
        
        int profileIndex = -1;
        do {
            System.out.println("Please enter the number for the SystemProfile you wish to restore: \n");
            System.out.flush();
            profileIndex = Integer.parseInt(inReader.readLine().trim());
        } while ( profileIndex < 0 || profileIndex >= profiles.length );
        RBSystemProfile theProfile = profiles[profileIndex];
        
        System.out.println("\n\nWe will schedule the previously created " +
                "ActivityTemplate to run on the host with IP: " 
                + rbhost.getIPAddress()
                + " and Description: " + rbhost.getDescription() + " Using the SystemProfile: " + theProfile.getDescription());
        
        /* Schedule Activity to use the RBOSRestoreTemplate we just created on user specified host */
        /* Note that for the RBOSRestoreTemplate, you must have the following in the parameter map:
         *   a RBSystemProfile object with key "Profile".
         *   
         *   e.g. HashMap params = new HashMap();
         *        params.put("Profile", yourProfile);
         */ 
        HashMap params = new HashMap();
        params.put("Profile", theProfile);
        RBActivity newAct = rbServer.registerActivity("Test Activity - OSRestore ", restoreTempl, 
                false, false, params, rbhost);
        System.out.println("\nCreated new activity that uses the RBOSRestoreTemplate " +
                "we just created to run immediately on host with IP: " 
                + rbhost.getIPAddress());
        printActivity(newAct);  
        
        System.out.println("Finished. Watch OS Restore activity on PXE client.");
        
        System.out.println("\nPress enter when the OS restore is finished and I \n" +
                "will remove the template to clean up.");
        inReader.readLine();
        
        /* Now remove the template */
        System.out.println("Removing the previously created template...");
        rbServer.removeTemplate(restoreTempl, true);
        System.out.println("Done.");
    }
    
    public static void printMap(Map map) {
        if (map != null) {
            Iterator keyIterator = (map.keySet()).iterator();
            while (keyIterator.hasNext()) {
                String key = (String) keyIterator.next();
                System.out.println("## key: " + key + " value: " + (map.get(key)).toString());
            }
        }
    }
    
    public static void printActivity(RBActivity activity) {
        System.out.println("\n### RBActivity - printing out ###");
        System.out.println("###  getDescription(): " + activity.getDescription());
        System.out.println("###  getDoneDate(): " + activity.getDoneDate());
        System.out.println("###  getExecDate(): " + activity.getExecDate());
        System.out.println("###  getExpireDate(): " + activity.getExpireDate());
        System.out.println("###  getProgress(): " + activity.getProgress());
        System.out.println("###  isSuspendOnError(): " + activity.isSuspendOnError());
    }
    
    public static void printActivityTarget(RBActivityTarget target) {
        System.out.println("\n### RBActivityTarget - printing out ###");
        System.out.println("###  getProgress(): " + target.getProgress());
        System.out.println("###  getStatus(): " + target.getStatus());
    }
}
