/* TestRBServerStatus.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2006
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestRBServerStatus.java#15 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;

import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.RBServerStatus;
import com.ibm.rbapi.exception.RBTimeoutException;

public class TestRBServerStatus {

    public static void main(String[] args) 
    throws UnknownHostException, RBServerException, IOException, RBTimeoutException
    {
        
        if (args.length < 3) {
            System.out.println("usage: TestRBServerStatus <ip> <port> <password>");
            return;
        }

        System.out.println("TestRBServerStatus starting.");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret); 
        System.out.println("Created rbServer.\n");
        
        /* Test isReady(), getVersions() */
        System.out.println("RBServer.isReady() returns: " + rbServer.isReady(15000));
        String vv[] = rbServer.getVersions();
        System.out.println("RBServer.getVersions() returns: " + vv[0] + " for rbapi.jar");
        System.out.println("RBServer.getVersions() returns: " + vv[1] + " for rbapi.onserver.jar\n");
        
        /* Test getStatus() */
        RBServerStatus rbServerStatus = rbServer.getStatus();
        System.out.println("RBserverStatus.getVersion() returns: " + rbServerStatus.getVersion());
        System.out.println("RBServerStatus.getUptime() returns: " + rbServerStatus.getUptime());
        System.out.println("RBServerStatus.getHostCount() returns: " + rbServerStatus.getHostCount());
        System.out.println("RBServerStatus.getHostLastMin() returns:  " + rbServerStatus.getHostLastMin());
        System.out.println("RBServerStatus.getHostLastHour() returns:  " + rbServerStatus.getHostLastHour());
        System.out.println("RBServerStatus.getHostLastDay() returns:  " + rbServerStatus.getHostLastDay());
        System.out.println("RBServerStatus.getLastMessage() returns:  " + rbServerStatus.getLastMessage());
        System.out.println("RBServerStatus.getSubnetCount() returns:  " + rbServerStatus.getSubnetCount());
        System.out.println("RBServerStatus.getRpcVersion() returns:  " + rbServerStatus.getRpcVersion());
        
        /* Test getRemboTime() */
        long remboTime = rbServer.getRemboTime();
        System.out.println("RBServer.getRemboTime() returns: " + remboTime);
        long javaTime = System.currentTimeMillis();
        System.out.println("Java time is: " + javaTime);
        long remboTimeMillis = remboTime*1000;
        long addForJavaTime = javaTime - remboTimeMillis;
        System.out.println("Difference: " + addForJavaTime);

        InetAddress local = InetAddress.getLocalHost();
        System.out.println("rbServer.getConsoleUrl("+local.getHostAddress()+") returns:  " + rbServer.getConsoleUrl(local,"ADMINISTRATORS"));
        
        System.out.println("\n\nThis program will now pause, so that you can restart the server.");
        System.out.println("After you press return, it will block until the server is ready.");
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        inReader.readLine();
        rbServer.isReady(0);
        System.out.println("\nServer is ready.");
                
        System.out.println("\n\nThis program will now pause, so that you can stop the server.");
        System.out.println("After you press return, it will wait 15 seconds and give up.");
        inReader.readLine();
        try {rbServer.isReady(15000);} catch (RBTimeoutException e) {};
        System.out.println("\nTest done.");
        System.out.println(rbServer.getStatistics());
        

    }

}
