/* TestSoftwarePackageCreation.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2006
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestSoftwarePackageCreation.java#2 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBActivityTargetResults;
import com.ibm.rbapi.RBEvent;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBHostRecs;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.RBSoftwarePackageParams;
import com.ibm.rbapi.RBWebExtensionStatus;
import com.ibm.rbapi.exception.RBTimeoutException;

public class TestSoftwarePackageCreation {

    public static void main(String[] args) 
    throws UnknownHostException, IOException, InterruptedException, 
    RBServerException, RBObjectNotFoundException 
    {
        if (args.length < 3) {
            System.out.println("usage: TestSoftwarePackageCreation <ip> <port> <password>");
            return;
        }

        System.out.println("TestSoftwarePackageCreation starting.");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String secret = args[2];
        RBServer rbServer = new RBServer(ip, port, secret); 
        System.out.println("Created rbServer.\n");
        
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        RBHostRecs hostRecs = rbServer.getHosts("", "");
        RBHost[] availHosts = hostRecs.subList(0, hostRecs.size()); 
        for (int i=0; i<availHosts.length; i++) {
            System.out.println(i + ") " + availHosts[i].getIPAddress()
                    + " MAC: " + availHosts[i].getMacAddress()
                    + " Description: " + availHosts[i].getDescription());
        }
        
        int hostIndex = -1;
        do {
            System.out.println("Please enter the number for the host you wish to use: \n");
            hostIndex = Integer.parseInt(inReader.readLine().trim());
        } while ( hostIndex < 0 || hostIndex >= hostRecs.size() );
        
        RBHost rbhost = availHosts[hostIndex];
        
        // Check if Web Extension is ready on that host
        System.out.println("Checking RBServer.isWebExtensionReady() for host: " + rbhost.getIPAddress() + "...");
        RBWebExtensionStatus wieStatus = null;
        for (int i=0; i<10; i++) {  // try at most 10 times (20 seconds)
            wieStatus = rbServer.isWebExtensionReady(rbhost.getIPAddress());
            if (wieStatus.isPresent()) {
                break;
            }
            Thread.sleep(2000);
        }
        System.out.println("WIE - isPresent(): " + wieStatus.isPresent());
        System.out.println("WIE - getWebExtensionVersion(): " + wieStatus.getWebExtensionVersion());
        System.out.println("WIE - getWebExtensionBuild(): " + wieStatus.getWebExtensionBuild());
        System.out.println("WIE - getServerVersion(): " + wieStatus.getServerVersion());
        System.out.println("WIE - getServerBuild(): " + wieStatus.getServerBuild());
        
        if (!wieStatus.isPresent()) {
            System.out.println("The Web Interface Extension is not detected on the target host.  " +
                    "Please ensure that it is started and run this test again.");
            System.exit(1);
        }
        
        int pkgType = 0;
        while (pkgType < 1 || pkgType > 6) {
            System.out.println("\n\nPlease enter the number of the Software Package type you \n" +
            "wish to create:");
            System.out.println("1) MSI package");
            System.out.println("2) Floppy disk image");
            System.out.println("3) Win PE image");
            System.out.println("4) RPM package");
            System.out.println("5) fileset creation package");
            System.out.println("6) Linux preboot image");
            // NOTE, there are more types - using these 6 as an example
            String packageType = inReader.readLine();
            pkgType = Integer.parseInt(packageType);
        }
        
        System.out.println("\n\nPlease enter the path to the directory containing your \n" +
           "file(s) that will be packaged.  This should be the path on the host.");
        String sourcePath = inReader.readLine();
        
        RBSoftwarePackageParams softPkgParams = null;
        switch (pkgType) {
            case 1:  
                System.out.println("Creating MSI package params"); 
                softPkgParams = 
                    RBSoftwarePackageParams.newMSICreationParams("test MSI pkg", 
                          "msi comment", sourcePath, 
                          "c:\\installTest\\testmsi", "");
                break;
            case 2:  
                System.out.println("Creating Floppy package params"); 
                softPkgParams = 
                    RBSoftwarePackageParams.newFloppyCreationParams("testFloppyPkg", 
                          "floppy comment", sourcePath);
                break;
            case 3:  
                System.out.println("Creating Win PE package params"); 
                softPkgParams = 
                    RBSoftwarePackageParams.newWinPECreationParams("testWinPEPkg", 
                            "winpe comment", sourcePath);
                break;
            case 4:
                System.out.println("Creating RPM package params");
                softPkgParams = 
                    RBSoftwarePackageParams.newRPMCreationParams("testRPMpkg", 
                            "rpm comment", sourcePath, "/testdest", "rpm -ivh");
                break;
            case 5:
                System.out.println("Creating new fileset creation package params");
                softPkgParams = 
                    RBSoftwarePackageParams.newFileSetCreationParams("testFileSetPkg", 
                            "fileset comment", sourcePath, "c:\\testdest", "cmd", 
                            false, false);
                break;
            case 6:
                System.out.println("Creating new Linux preboot package params");
                softPkgParams = 
                    RBSoftwarePackageParams.newLinuxPECreationParams("testLinuxPEPkg", 
                            "Linux kernel and initial ramdisk", sourcePath, "bootimg", "ramdisk");
                break;
            default: 
                System.out.println("Invalid selection.  Exiting.");
                System.exit(-1);
        }
        
        /*
         * We can decide if we want to remove the installation files after
         * installing the software package.
         */
        System.out.println("Do you wish to remove the install files after " +
                "installation of this SoftwarePackage? \n" +
                "(Enter y for yes, default is no): [n] ");
        String removeFilesString = inReader.readLine().trim().toLowerCase();
        boolean removeFiles = false;
        if (removeFilesString.equals("y") || removeFilesString.equals("yes")) {
            removeFiles = true;
        }
        /* 
         * Now let's set in the software package params if we want to remove
         * files after install or not.
         */
        softPkgParams.setRemoveFiles(removeFiles);
        
   
        System.out.println("Create softwarePackageParams done.");
        printSoftwarePackageParams(softPkgParams);
        
        System.out.println("\n\nPerforming a probeSoftwarePackageCreation on the target...");
        RBActivityTargetResults probeResults = null;
        for (int i=0; i<2; i++) {
            System.out.println("Probe attempt #" + i);
            try {
                probeResults = rbServer.probeSoftwarePackageCreation(rbhost, softPkgParams, 30+(i*10));
                break;
            } catch (RBTimeoutException timeEx) {
                timeEx.printStackTrace();
                System.out.println("Probe timed out.");
            }
        }
        if (probeResults == null) {
            System.out.println("Probe is not completing within the timeout.  Exiting.");
            System.exit(1);
        }
        System.out.println("\nProbe done.");
        
        System.out.println("status of probe: " + probeResults.getStatus());
        if (!probeResults.getStatus().equals(RBActivityTarget.COMPLETED)) {
            System.out.println("Probe did not complete successfully.  Printing out the error:");
            printMap(probeResults.getError());
            System.out.println("\n\nExiting.");
            System.exit(-1);
        }
            
        System.out.println("Printing out updated SoftwarePackageParams from probe: ");
        printSoftwarePackageParams(softPkgParams);
        
        System.out.println("\n\nNow we will create the software package...");
        Date now = new Date();
        Date stop = new Date(now.getTime()+1000*3600);
        Date expire = new Date(now.getTime()+1000*3600*24);
        RBActivity act = rbServer.registerSoftwarePackageCreationActivity("TestImageCapture", 
                true, null, rbhost, softPkgParams, 
                now, stop, expire);
        
        /*
         * This section here refreshes the activity target to see if the activity
         * is complete or not.  This is done just to code something quickly, but 
         * is not very efficient as the polling results in making queries to the 
         * server quite often.  For an example of better activity event monitoring,
         * see the TestRBEvents.java example class, or the TestStressSmart.java class.
         */
        Date lastTime = new Date();
        boolean foundCompletedEvent = false;
        while (!foundCompletedEvent) {
            Thread.sleep(8000);
            System.out.println("Getting all events from server from time: " + lastTime);
            RBEvent[] events =  act.getEventsFromServer(lastTime);
            RBEvent e = null;
            for (int i=0; i<events.length; i++) {
                e = events[i];

                System.out.println("Event: Progress=" + e.getProgress() + 
                        " Progress Msg=" + e.getLastProgressMsg() + 
                        " Status=" + e.getStatus());
                
                // Let's get information about events that are completed or failed
                if ( e.getStatus().equals(RBActivityTarget.COMPLETED) ||
                        e.getStatus().equals(RBActivityTarget.FAILED)) {
                    System.out.println("Getting corresponding RBActivityTarget object");
                    RBActivityTarget ourActTarget = e.getRBActivityTarget();
                    System.out.println("Activity Target: Progress=" + ourActTarget.getProgress() + 
                            " Progress Msg=" + ourActTarget.getLastProgressMsg() + 
                            " Status=" + ourActTarget.getStatus());
                    
                    // Print out the specific results for the activity target
                    RBActivityTargetResults results = ourActTarget.getActivityTargetResults();
                    System.out.println("### Results for our software creation activity: ");
                    System.out.println("Status: " + results.getStatus());
                    RBSoftwarePackageParams resultPkgParams = results.getSoftwarePackageParameters();
                    System.out.println("\nResult softwarePackage params: ");
                    printSoftwarePackageParams(resultPkgParams);

                    foundCompletedEvent = true;
                    System.out.println("Exiting loop.");
                }
            }
            if (e != null) {
                lastTime = e.getUpdateTime();
            } else {
                System.out.println("No events found.");
            }
        }
        
        System.out.print("\nTestSoftwarePackageCreation done.");
    }
    
    private static void printMap(Map map) {
        System.out.println("Printing Error from result: " );
        Set keys = map.keySet();
        Iterator keyIt = keys.iterator();
        while (keyIt.hasNext()) {
            String key = (String) keyIt.next();
            System.out.println("    key: " + key
                    + " -> " + (String) map.get(key));
        }
    }
    
    private static void printSoftwarePackageParams(RBSoftwarePackageParams softPkgParams) {
        System.out.println("\n### Printing out software package Params ###");
        System.out.println("Package Description: " + softPkgParams.getPkgDescr());
        System.out.println("Package Comment: " + softPkgParams.getComment());
        System.out.println("Destination Path: " + softPkgParams.getDestPath());
        System.out.println("Driver Class: " + softPkgParams.getDriverClass());
        System.out.println("Driver Version: " + softPkgParams.getDriverVersion());
        System.out.println("MSI file name: " + softPkgParams.getMsiFileName());
        System.out.println("MSI file names list: " + softPkgParams.getMsiFileNamesList());
        System.out.println("Package file name: " + softPkgParams.getPackageFileName());
        System.out.println("Product name: " + softPkgParams.getProductName());
        System.out.println("Product version: " + softPkgParams.getProductVersion());
        System.out.println("Source path: " + softPkgParams.getSourcePath());
        System.out.println("Vendor: " + softPkgParams.getVendor());
        System.out.println("Remove Files after install: " + softPkgParams.isRemoveFiles());
    }

}
