/* TestToolkitProfileCreation.java
 *
 * -----------------------------------------------------------------
 * Licensed Materials - Property of IBM
 * 5724Q99
 * (C) Copyright IBM Corp. 1998, 2007
 * All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 * -----------------------------------------------------------------
 * $Id: //depot/rbo/src/packages/ibmee/javaapi/examples/TestToolkitProfileCreation.java#1 $
 */
package examples;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;

import com.ibm.rbapi.RBActivity;
import com.ibm.rbapi.RBActivityTarget;
import com.ibm.rbapi.RBEvent;
import com.ibm.rbapi.RBHost;
import com.ibm.rbapi.RBObjectNotFoundException;
import com.ibm.rbapi.RBProfileCreationParams;
import com.ibm.rbapi.RBServer;
import com.ibm.rbapi.RBServerException;
import com.ibm.rbapi.exception.RBTimeoutException;

public class TestToolkitProfileCreation {

    public static void main(String[] args) 
    throws UnknownHostException, RBServerException, RBObjectNotFoundException, 
    IOException, InterruptedException, RBTimeoutException
    {
        if (args.length < 3) {
            System.out.println("usage: TestToolkitProfileCreation <ip> <port> <password>");
            return;
        }
        System.out.println("TestToolkitProfileCreation starting...");
        InetAddress ip = InetAddress.getByName(args[0]);
        int port = Integer.parseInt(args[1]);
        String token = args[2];
        RBServer rbServer = new RBServer(ip, port, token); 
        System.out.println("Created rbServer.");
        
        rbServer.isReady(15000);
        
        System.out.println("\nEnter the name of your toolkit image you want to convert to\n" +
                "a system profile - The image must be in the <datadir>/global/hdimages\n" +
                "directory with the normal naming convention: \n");
        BufferedReader inReader = new BufferedReader(new InputStreamReader(System.in));
        String tkitImageName = inReader.readLine().trim();
        
        System.out.println("\nWe will now run a toolkit profile probe on the image: " 
                + tkitImageName);
        
        RBProfileCreationParams profCreateParams = null;
        boolean successful = false;
        int count = 0;
        int timeout = 40;
        while (count <= 3) {
            try {
                count++; 
                System.out.println("Probing the toolkit profile with timeout of: " + timeout + " seconds."); 
                profCreateParams = rbServer.probeToolkitProfile(tkitImageName, timeout);
                if(profCreateParams.getStatus().equals("COMPLETED")) {
                    System.out.println("Probe completed successfully.");
                    successful = true;
                } else {
                    System.out.println("Unexpected probe status - Status returned is: " + profCreateParams.getStatus());
                    printMap(profCreateParams.getError());
                }
                break;
            } catch (RBTimeoutException tme) {
                System.out.println("Probe timed out.");
                timeout = timeout + 20;
            }
        }
        
        if (!successful) {
            System.out.println(
                    "Probe did not complete successfully - ensure you have specified a " +
                    "valid toolkit image name, and that it resides in the proper " +
                    "location on disk, then retry the test.");
            System.exit(1);
        }
        
        
        String description = "Test Tkit profile";
        System.out.println("Now we will create the profile from the toolkit image...");
        System.out.println("Will set the description of the system profile to: '"+description+"'.");
        System.out.println("Will set allow no sysprep to true - so we can still convert toolkit images " +
                "that are not sysprepped.");
        profCreateParams.setDescription(description);
        profCreateParams.setAllowNoSysprep(true);
        
        Date now = new Date();
        Date later = new Date(now.getTime()+(1000*3600*24));
        RBActivity tkitAct = rbServer.registerToolkitProfileCreation(
                "Test toolkit prof creation", true, 
                null, profCreateParams, now, later, later);
        
        checkForActivityFinished(tkitAct, now); 
        
        // Before leaving, display RPC statistics            
        System.out.println("Stats for TestToolkitProfileCreation:");
        System.out.println(rbServer.getStatistics());
        System.out.println("Done.");
    }
    

    static void checkForActivityFinished(RBActivity act, Date startTime) 
    throws RBServerException, RBObjectNotFoundException, InterruptedException 
    {
        /*
         * This section here refreshes the activity target to see if the activity
         * is complete or not.  This is done just to code something quickly, but 
         * is not very efficient as the polling results in making queries to the 
         * server quite often.  For an example of better activity event monitoring,
         * see the TestRBEvents.java example class, or the TestStressSmart.java class.
         */
        int count = 0;  // Loop through and exit when we find the first finished event
        Date lastTime = startTime;
        boolean foundCompletedEvent = false;
        while (!foundCompletedEvent && count < 20 ) {
            System.out.println("Getting all events from server from time: " + lastTime);
            RBEvent[] events = act.getEventsFromServer(lastTime);

            RBEvent e = null;
            for (int i=0; i<events.length; i++) {
                e = events[i];

                System.out.println("Event: Progress=" + e.getProgress() + " Status=" + e.getStatus());

                // Let's get information about events that are completed or failed
                if ( e.getStatus().equals(RBActivityTarget.COMPLETED) ||
                        e.getStatus().equals(RBActivityTarget.FAILED)) {
                    System.out.println("Event status is: " + e.getStatus() + " Now let's get some more info from the server\n"); 

                    // Different ways to get information, here are some examples (you would probably not use these all at the same time):
                    System.out.println("Getting corresponding RBHost object");
                    RBHost rbHost = e.getRBHost();
                    System.out.println(" The event that just completed was run on host: " 
                            + rbHost.getDescription() 
                            + ", " + rbHost.getIPAddress()
                            + ", " + rbHost.getMacAddress());

                    System.out.println("Getting corresponding RBActivity object");
                    RBActivity rbAct = e.getRBActivity();
                    System.out.println(" The event that just completed was part of Activity: " + rbAct.getDescription());

                    System.out.println("Getting corresponding RBActivityTarget object");
                    RBActivityTarget rbActTarget = e.getRBActivityTarget();
                    System.out.println("Results: params -- ");
                    printMap(rbActTarget.getParams());
                    System.out.println("Results: errors -- ");
                    printMap(rbActTarget.getError());

                    // Also note that if you already have instances of an RBActivity, RBActivityTarget or RBHost, you can use the RBEvent.belongsTo* methods
                    // to see if they belong to your already instantiated objects, rather than going to the server to retrieve new ones, as in the 
                    // code above.

                    foundCompletedEvent = true;
                    System.out.println("Exiting loop.");
                }
            }
            
            if (e != null) {
                lastTime = e.getUpdateTime();
            } else {
                System.out.println("No events found.");
            }
            count++;
            Thread.sleep(4000);
        }
    }
    
    public static void printMap(Map map) {
        if (map != null) {
            Iterator keyIterator = (map.keySet()).iterator();
            while (keyIterator.hasNext()) {
                String key = (String) keyIterator.next();
                System.out.println("## key: " + key + " value: " + (map.get(key)).toString());
            }
        }
    }
}
